#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var child_process = require("child_process");
var glob = require("glob");
var path = require("path");
var yaml = require("js-yaml");
var program = require("commander");
var src;
var repoRelativePath;
var dest = 'docs-ref-autogen';
var packageWhiteList = [];
var packageBlackList = [];
var config;
var pjson = require('../package.json');
var envPath = process.env['APPDATA'] + "/npm/node_modules/ts-reference-ci-scripts";
program
    .version("v" + pjson.version)
    .description('A tool to go through source code and convert it to yaml data model for docfx.')
    .option('--config [value]', 'The config file which will defined more specefied behaviour for the tool')
    .option('--whiteList [value]', 'White list some package name for extraction.')
    .option('--blackList [value]', 'Black list some package name for extraction. Black list will override whitelist.')
    .option('--destPath [value]', 'Path to output the autogen files.')
    .arguments('<sourceFolder> [repoConfigPath]')
    .action(function (sourceFolder, repoConfig) {
    src = sourceFolder;
    repoRelativePath = repoConfig;
})
    .parse(process.argv);
if (!src) {
    console.log('Error: The source file folder is not specified!');
    program.help();
}
if (program.whiteList) {
    try {
        var tempList = JSON.parse(program.whiteList);
        if (tempList && tempList.length) {
            packageWhiteList = tempList;
        }
    }
    catch (e) {
        console.log('Invalid json format of white list, ignored!');
    }
}
if (program.blackList) {
    try {
        var tempList = JSON.parse(program.blackList);
        if (tempList && tempList.length) {
            packageBlackList = tempList;
        }
    }
    catch (e) {
        console.log('Invalid json format of black list, ignored!');
    }
}
if (program.config) {
    try {
        config = JSON.parse(program.config);
    }
    catch (e) {
        console.log('Invalid json format of config, ignored!');
    }
}
if (program.destPath) {
    dest = program.destPath + "/" + dest;
}
// 1. prepare
fs.removeSync(dest);
fs.mkdirpSync(dest);
var tsconfigs = glob.sync(path.join(src, '**/tsconfig.json'));
var toc = [];
tsconfigs.forEach(function (tsconfig) {
    var packagePath = tsconfig.replace('tsconfig.json', 'package.json');
    generatePackageDoc(packagePath, dest);
});
toc = JSON.parse(JSON.stringify(toc));
fs.writeFileSync(dest + "/toc.yml", yaml.safeDump(toc));
process.exit(0);
function generatePackageDoc(packagePath, dest) {
    var dir = path.dirname(packagePath);
    if (!fs.existsSync(packagePath)) {
        return;
    }
    var packageName = fs.readJsonSync(packagePath).name;
    if (packageWhiteList.length === 0 || packageWhiteList.indexOf(packageName) >= 0) {
        if (packageBlackList.indexOf(packageName) >= 0) {
            return;
        }
        console.log(packageName);
        console.log(dir);
        var sourceCodeBasePath = dir + "/src";
        if (fs.existsSync(sourceCodeBasePath)) {
            var packageDest = dest + "/" + packageName;
            fs.mkdirpSync(packageDest);
            try {
                child_process.execSync("typedoc --json " + dir + "/api.json " + dir + "/src --module commonjs --includeDeclarations --ignoreCompilerErrors");
                var basePath = sourceCodeBasePath.replace(src + "/", '');
                if (config && config.source_repo) {
                    child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repo.url + " --sourceBranch " + config.source_repo.branch + " --basePath " + basePath);
                }
                else if (config && config.source_repos) {
                    var folder = dir.split('/')[1];
                    var processed = false;
                    if (folder) {
                        for (var index in config.source_repos) {
                            if (folder === config.source_repos[index].folder) {
                                child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repos[index].url + " --sourceBranch " + config.source_repos[index].branch + " --basePath " + basePath);
                                processed = true;
                                break;
                            }
                        }
                    }
                    if (!processed) {
                        child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repos[0].url + " --sourceBranch " + config.source_repos[0].branch + " --basePath " + basePath);
                    }
                }
                else {
                    child_process.execSync("type2docfx " + dir + "/api.json " + packageDest);
                }
                if (fs.existsSync(packageDest + "/toc.yml")) {
                    var subToc = yaml.safeLoad(fs.readFileSync(packageDest + "/toc.yml").toString());
                    toc.push(subToc[0]);
                    fs.removeSync(packageDest + "/toc.yml");
                }
            }
            catch (e) {
                console.log(packageName + ": TypeDoc Parse Error.");
            }
        }
        else {
            console.log("No source file for " + packageName);
        }
    }
}
